;;; Copyright (c) 1996-2002 Brian D. Carlstrom

;;; primitive
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Debugging
(define break-point (new "bdc.scheme.procedure.BreakPoint"))
(define error (new "bdc.scheme.procedure.Error"))

;; Syntax extension
(define define-rewriter (new "bdc.scheme.procedure.DefineRewriter"))
(define gensym (new "bdc.scheme.procedure.GenSym"))
(define-rewriter 'let (new "bdc.scheme.compiler.Let2Application"))
(define-rewriter 'cond (new "bdc.scheme.compiler.Cond2If"))
(define-rewriter 'or (new "bdc.scheme.compiler.Or2If"))
(define-rewriter 'and (new "bdc.scheme.compiler.And2If"))
(define-rewriter 'do (new "bdc.scheme.compiler.Do2Until"))

;; Predicates
(define eq? (new "bdc.scheme.procedure.Eq"))
(define instanceof? (new "bdc.scheme.procedure.InstanceOf"))

;; Math
(define = (new "bdc.scheme.procedure.Equals"))
(define < (new "bdc.scheme.procedure.LessThan"))
(define + (new "bdc.scheme.procedure.Plus"))
(define - (new "bdc.scheme.procedure.Minus"))
(define * (new "bdc.scheme.procedure.Times"))
(define / (new "bdc.scheme.procedure.Divide"))
(define remainder (new "bdc.scheme.procedure.Remainder"))
(define floor (new "bdc.scheme.procedure.Floor"))
(define exp (new "bdc.scheme.procedure.Exp"))
(define sin (new "bdc.scheme.procedure.Sin"))
(define cos (new "bdc.scheme.procedure.Cos"))
(define tan (new "bdc.scheme.procedure.Tan"))
(define asin (new "bdc.scheme.procedure.Asin"))
(define acos (new "bdc.scheme.procedure.Acos"))
(define atan (new "bdc.scheme.procedure.Atan"))
(define sqrt (new "bdc.scheme.procedure.Sqrt"))

;; Vector
(define make-vector (new "bdc.scheme.procedure.MakeVector"))
(define vector-length (new "bdc.scheme.procedure.VectorLength"))
(define vector-ref (new "bdc.scheme.procedure.VectorRef"))
(define vector-set! (new "bdc.scheme.procedure.VectorSet"))

;; String
(define make-string (new "bdc.scheme.procedure.MakeString"))
(define string-length (new "bdc.scheme.procedure.StringLength"))
(define string-ref (new "bdc.scheme.procedure.StringRef"))
(define string-set! (new "bdc.scheme.procedure.StringSet"))
(define string->number (new "bdc.scheme.procedure.String2Number"))
(define string->symbol (new "bdc.scheme.procedure.String2Symbol"))

;; Pair
(define cons (new "bdc.scheme.procedure.Cons"))
(define car (new "bdc.scheme.procedure.Car"))
(define cdr (new "bdc.scheme.procedure.Cdr"))
(define set-car! (new "bdc.scheme.procedure.SetCar"))
(define set-cdr! (new "bdc.scheme.procedure.SetCdr"))

;; Char
(define char=? (new "bdc.scheme.procedure.CharEqual"))
(define char->integer (new "bdc.scheme.procedure.Char2Integer"))
(define integer->char (new "bdc.scheme.procedure.Integer2Char"))
(define char-whitespace? (new "bdc.scheme.procedure.CharWhitespace"))
(define char-lower-case? (new "bdc.scheme.procedure.CharLowerCase"))
(define char-upper-case? (new "bdc.scheme.procedure.CharUpperCase"))
(define char-numeric? (new "bdc.scheme.procedure.CharNumeric"))
(define char-alphabetic? (new "bdc.scheme.procedure.CharAlphabetic"))
(define char-upcase (new "bdc.scheme.procedure.CharUpcase"))
(define char-downcase (new "bdc.scheme.procedure.CharDowncase"))

;; Control features
(define apply (new "bdc.scheme.procedure.Apply"))
(define dynamic-wind (new "bdc.scheme.procedure.DynamicWind"))
(define eval (new "bdc.scheme.procedure.Eval"))

;; I/O
(define open-input-file (new "bdc.scheme.procedure.OpenInputFile"))
(define open-output-file (new "bdc.scheme.procedure.OpenOutputFile"))
(define close-input-port (new "bdc.scheme.procedure.CloseInputPort"))
(define close-output-port (new "bdc.scheme.procedure.CloseOutputPort"))
(define read-char (new "bdc.scheme.procedure.ReadChar"))
(define peek-char (new "bdc.scheme.procedure.PeekChar"))
(define write-char (new "bdc.scheme.procedure.WriteChar"))
(define read (new "bdc.scheme.procedure.Read"))
(define write (new "bdc.scheme.procedure.Write"))
(define read-line (new "bdc.scheme.procedure.ReadLine"))
(define display (new "bdc.scheme.procedure.Display"))
(define eof-object? (new "bdc.scheme.procedure.EOFObject"))
(define char-ready? (new "bdc.scheme.procedure.CharReady"))
(define load (new "bdc.scheme.procedure.Load"))

;; ports
(define current-input-port (new "bdc.scheme.procedure.CurrentInputPort"))
(define current-output-port (new "bdc.scheme.procedure.CurrentOutputPort"))
(define current-error-port (new "bdc.scheme.procedure.CurrentErrorPort"))
(define set-current-input-port (new "bdc.scheme.procedure.SetCurrentInputPort"))
(define set-current-output-port (new "bdc.scheme.procedure.SetCurrentOutputPort"))
(define set-current-error-port (new "bdc.scheme.procedure.SetCurrentErrorPort"))

;; transcripts
(define transcript-on (new "bdc.scheme.procedure.TranscriptOn"))
(define transcript-off (new "bdc.scheme.procedure.TranscriptOff"))

;; call/cc
(define call-with-current-continuation (new "bdc.scheme.procedure.CallCC"))

;; Exceptions
(define catch (new "bdc.scheme.procedure.Catch"))
(define throw (new "bdc.scheme.procedure.Throw"))

;; Reflection
(define class-for-name (new "bdc.scheme.procedure.ClassForName"))
(define class-get-method (new "bdc.scheme.procedure.ClassGetMethod"))
(define method-invoke (new "bdc.scheme.procedure.MethodInvoke"))

(define Boolean.class   (class-for-name "java.lang.Boolean"))
(define Byte.class      (class-for-name "java.lang.Byte"))
(define Character.class (class-for-name "java.lang.Character"))
(define Double.class    (class-for-name "java.lang.Double"))
(define Float.class     (class-for-name "java.lang.Float"))
(define Integer.class   (class-for-name "java.lang.Integer"))
(define Long.class      (class-for-name "java.lang.Long"))
(define Short.class     (class-for-name "java.lang.Short"))
(define Void.class      (class-for-name "java.lang.Void"))

(define Class.class     (class-for-name "java.lang.Class"))
(define Number.class    (class-for-name "java.lang.Number"))
(define Object.class    (class-for-name "java.lang.Object"))
(define String.class    (class-for-name "java.lang.String"))

(define Array.class       (class-for-name "java.lang.reflect.Array"))
(define Constructor.class (class-for-name "java.lang.reflect.Constructor"))
(define Field.class       (class-for-name "java.lang.reflect.Field"))
(define Method.class      (class-for-name "java.lang.reflect.Method"))

(define PushbackReader.class (class-for-name "java.io.PushbackReader"))
(define PrintWriter.class    (class-for-name "java.io.PrintWriter"))

(define List.class    (class-for-name "java.util.List"))

(define Pair.class      (class-for-name "bdc.scheme.Pair"))
(define Symbol.class    (class-for-name "bdc.scheme.Symbol"))
(define Procedure.class (class-for-name "bdc.scheme.expression.Procedure"))

(define CharArray.class (class-for-name "[C"))

(define (make-method class name . parameterTypes)
  (let ((method (apply class-get-method class name parameterTypes)))
    (lambda x (apply method-invoke method x))))

(define (make-static-method class name . parameterTypes)
  (let ((method (apply class-get-method class name parameterTypes)))
    (lambda x (apply method-invoke method '() x))))

(define class-get-declared-field
  (make-method Class.class "getDeclaredField" String.class))

(define class-get-public-field
  (make-method Class.class "getField" String.class))

(define applet #f)

(define class-get-field '())
(set! class-get-field
      (lambda (class name)
        (catch (lambda ()
                 (class-get-declared-field class name))
               "java.lang.SecurityException"
               (lambda (e)
                 (set! applet #t)
                 (set! class-get-field class-get-public-field)
                 (class-get-public-field class name)))))

(define field-get (make-method Field.class "get" Object.class))

(define field-set (make-method Field.class "set" Object.class Object.class))

(define (make-field-getter class name)
  (let ((field (class-get-field class name)))
    (lambda (obj)
      (field-get field obj))))

(define (make-field-setter class name)
  (let ((field (class-get-field class name)))
    (lambda (obj value)
      (field-set field obj value))))

(define Boolean.TYPE   ((make-field-getter Boolean.class   "TYPE") '()))
(define Byte.TYPE      ((make-field-getter Byte.class      "TYPE") '()))
(define Character.TYPE ((make-field-getter Character.class "TYPE") '()))
(define Double.TYPE    ((make-field-getter Double.class    "TYPE") '()))
(define Float.TYPE     ((make-field-getter Float.class     "TYPE") '()))
(define Integer.TYPE   ((make-field-getter Integer.class   "TYPE") '()))
(define Long.TYPE      ((make-field-getter Long.class      "TYPE") '()))
(define Short.TYPE     ((make-field-getter Short.class     "TYPE") '()))
(define Void.TYPE      ((make-field-getter Void.class      "TYPE") '()))

(define object-get-class (make-method Object.class "getClass"))

(define array-new
  (make-static-method Array.class "newInstance" Class.class Integer.TYPE))

(define array-get-length
  (make-static-method Array.class "getLength" Object.class))

(define array-get
  (make-static-method Array.class "get" Object.class Integer.TYPE))

(define array-set
  (make-static-method Array.class
                      "set"
                      Object.class
                      Integer.TYPE
                      Object.class))

(define (array-class class)
  (object-get-class (array-new class 0)))

(define ClassArray.class (array-class Class.class))

(define class-get-constructor
  (make-method Class.class "getDeclaredConstructor" ClassArray.class))

(define ObjectArray.class (array-class Object.class))

(define constructor-new
  (make-method Constructor.class "newInstance" ObjectArray.class))

(define (make-constructor class . parameters)
  (let ((constructor (class-get-constructor class
                                            (list->array parameters
                                                         Class.class))))
    (lambda initArgs
      (constructor-new constructor (list->array initArgs Object.class)))))

(define (list->array lst class)
  (let ((c (length lst)))
    (let ((a (array-new class c)))
      (do ((i 0 (+ i 1))
           (l lst (cdr l)))
          ((= i c) a)
        (array-set a i (car l))))))

;;; derived
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (unspecific) (if #f #f))

(define (not x) (if x #f #t))

(define (pair? x)        (instanceof? x Pair.class))
(define (symbol? x)      (instanceof? x Symbol.class))
(define (procedure? x)   (instanceof? x Procedure.class))
(define (vector? x)      (instanceof? x List.class))
(define (input-port? x)  (instanceof? x PushbackReader.class))
(define (output-port? x) (instanceof? x PrintWriter.class))
(define (char? x)        (instanceof? x Character.class))
(define (number? x)      (instanceof? x Number.class))
(define (integer? x)     (instanceof? x Integer.class))
(define (real? x)    (or (instanceof? x Double.class)
                         (instanceof? x Integer.class)))
(define (string? x)  (or (instanceof? x String.class)
                         (instanceof? x CharArray.class)))

(define (null? x) (eq? x '()))
(define (boolean? x) (or (eq? x #t) (eq? x #f)))
(define (exact? x) #t)

(define (eqv? x y)
  (or (eq? x y)
      (and (number? x)
           (number? y)
           (eq? (exact? x) (exact? y))
           (= x y))
      (and (char? x)
           (char? y)
           (char=? x y))))

(define (equal? obj1 obj2)
  (cond ((eqv? obj1 obj2) #t)
        ((pair? obj1)
         (and (pair? obj2)
              (equal? (car obj1) (car obj2))
              (equal? (cdr obj1) (cdr obj2))))
        ((string? obj1)
         (and (string? obj2)
              (string=? obj1 obj2)))
        ((vector? obj1)
         (and (vector? obj2)
              (let ((z (vector-length obj1)))
                (and (= z (vector-length obj2))
                     (do ((i 0 (+ i 1))
                          (result '()))
                         ((boolean? result) result)
                       (cond ((= i z) (set! result #t))
                             ((equal? (vector-ref obj1 i) (vector-ref obj2 i)))
                             (else (set! result #f))))))))
        (else
         (equals? obj1 obj2))))

(define (> . n) (apply < (reverse n)))
(define (make-compare-or-equal test)
  (lambda n
    (do ((a      n  (cdr a))
         (b (cdr n) (cdr b))
         (result '()))
        ((boolean? result) result)
      (cond ((test (car a) (car b))
             (set! result #f))
            ((null? (cdr b))
             (set! result #t))))))
(define <= (make-compare-or-equal >))
(define >= (make-compare-or-equal <))

(define (max first . rest)
  (let ((result first))
    (for-each (lambda (x)
                (if (< result x)
                    (set! result x)))
              rest)
    result))

(define (min first . rest)
  (let ((result first))
    (for-each (lambda (x)
                (if (< x result)
                    (set! result x)))
              rest)
    result))

(define (abs n) (if (< n 0) (- 0 n) n))

(define (zero? x) (= x 0))
(define (positive? x) (< 0 x))
(define (negative? x) (< x 0))

(define (even? n) (= 0 (remainder n 2)))
(define (odd? n) (not (even? n)))

(define (caar   x) (car (car x)))
(define (cadr   x) (car (cdr x)))
(define (cdar   x) (cdr (car x)))
(define (cddr   x) (cdr (cdr x)))

(define (caaar  x) (caar (car x)))
(define (caadr  x) (caar (cdr x)))
(define (cadar  x) (cadr (car x)))
(define (caddr  x) (cadr (cdr x)))
(define (cdaar  x) (cdar (car x)))
(define (cdadr  x) (cdar (cdr x)))
(define (cddar  x) (cddr (car x)))
(define (cdddr  x) (cddr (cdr x)))

(define (caaaar x) (caaar (car x)))
(define (caaadr x) (caaar (cdr x)))
(define (caadar x) (caadr (car x)))
(define (caaddr x) (caadr (cdr x)))
(define (cadaar x) (cadar (car x)))
(define (cadadr x) (cadar (cdr x)))
(define (caddar x) (caddr (car x)))
(define (cadddr x) (caddr (cdr x)))
(define (cdaaar x) (cdaar (car x)))
(define (cdaadr x) (cdaar (cdr x)))
(define (cdadar x) (cdadr (car x)))
(define (cdaddr x) (cdadr (cdr x)))
(define (cddaar x) (cddar (car x)))
(define (cddadr x) (cddar (cdr x)))
(define (cdddar x) (cdddr (car x)))
(define (cddddr x) (cdddr (cdr x)))

(define (list . l) l)

(define (length l)
  (let ((n 0))
    (for-each (lambda (x) (set! n (+ n 1)))
              l)
    n))

(define (reverse! l)
  (cond ((null? l) l)
        ((null? (cdr l)) l)
        (else
         (do ((prev '())
              (cur l)
              (next (cdr l)))
             ((null? cur) prev)
           (set-cdr! cur prev)
           (set! prev cur)
           (set! cur next)
           (if (pair? cur) (set! next (cdr cur)))))))

(define (reverse l)
  (do ((new '() (cons (car l) new))
       (l l (cdr l)))
      ((null? l) new)))

(define (length l)
  (do ((i 0 (+ i 1))
       (l l (cdr l)))
      ((null? l) i)))

(define (append . lists)
  (cond ((null? lists) lists)
        ((null? (cdr lists)) (car lists))
        (else
         (let ((lists (reverse! lists)))
           (do ((lists (cdr lists) (cdr lists))
                (result (car lists) (append2 (car lists) result)))
               ((null? lists) result))))))

(define (append2 l1 l2)
  (cond ((null? l1) l2)
        (else
         (let ((last (reverse l1)))
           (let ((first (reverse! last)))
             (set-cdr! last l2)
             first)))))

(define (append-reverse list seed)
  (if (null? list)
      seed
      (append-reverse (cdr list) (cons (car list) seed))))

(define (filter pred lst)
  (let ((result '()))
    (for-each (lambda (x)
                (if (pred x)
                    (set! result (cons x result))))
              lst)
    (reverse result)))

(define (list-tail x k)
  (if (zero? k)
      x
      (list-tail (cdr x) (- k 1))))

(define (list-ref l k)
  (car (list-tail l k)))

(define (mem pred)
  (lambda (obj l)
    (do ((l l (cdr l)))
        ((or (null? l)
             (pred obj (car l)))
         (if (null? l)
             #f
             l)))))

(define memq   (mem eq?))
(define memv   (mem eqv?))
(define member (mem equal?))

(define (ass pred)
  (lambda (obj l)
    (do ((l l (cdr l)))
        ((or (null? l)
             (pred obj (caar l)))
         (if (null? l)
             #f
             (car l))))))

(define assq  (ass eq?))
(define assv  (ass eqv?))
(define assoc (ass equal?))

(define (list? l)
  (do ((l l )
       (lag l)
       (result '()))
      ((boolean? result) result)
    (cond ((null? l)
           (set! result #t))
          ((pair? l)
           (cond ((null? (cdr l))
                  (set! result #t))
                 ((and (pair? (cdr l))
                       (not (eq? (cdr l) lag)))
                  (set! l (cddr l))
                  (set! lag (cdr lag)))
                 (else
                  (set! result #f))))
          (else
           (set! result #f)))))

(define (char<? x y) (< (char->integer x) (char->integer y)))
(define (char>? x y) (char<? y x))
(define (char>=? x y) (not (char<? x y)))
(define (char<=? x y) (not (char>? x y)))

(define (char-ci-compare pred)
  (lambda (c1 c2) (pred (char-upcase c1) (char-upcase c2))))
(define char-ci=? (char-ci-compare char=?))
(define char-ci<? (char-ci-compare char<?))
(define char-ci<=? (char-ci-compare char<=?))
(define char-ci>? (char-ci-compare char>?))
(define char-ci>=? (char-ci-compare char>=?))

(define (string . rest)
  (list->string rest))

(define (substring s start end)
  (let ((new-string (make-string (- end start) #\space)))
    (do ((i start (+ i 1))
         (j 0 (+ j 1)))
        ((= i end) new-string)
      (string-set! new-string j (string-ref s i)))))

(define (string-append  . strings)
  (let ((len (do ((strings strings (cdr strings))
                  (len 0))
                 ((null? strings) len)
               (set! len (+ len (string-length (car strings)))))))
    (let ((new-string (make-string len #\space)))
      (do ((s strings)
           (k 0))
          ((null? s) new-string)
        (let ((string (car s)))
          (let ((l (string-length string)))
            (do ((j 0 (+ j 1))
                 (i k (+ i 1)))
                ((= j l) (set! s (cdr s)) (set! k i))
              (string-set! new-string i (string-ref string j)))))))))

(define (string->list v)
  (let ((z (string-length v)))
    (do ((i (- z 1) (- i 1))
         (l '() (cons (string-ref v i) l)))
        ((< i 0) l))))


(define (list->string l)
  (let ((v (make-string (length l) #\space)))
    (do ((i 0 (+ i 1))
         (l l (cdr l)))
        ((null? l) v)
      (string-set! v i (car l)))))

(define (string-copy s)
  (let ((z (string-length s)))
    (let ((copy (make-string z #\space)))
      (do ((i 0 (+ i 1)))
          ((= i z) copy)
        (string-set! copy i (string-ref s i))))))

(define (string-fill! v x)
  (let ((z (string-length v)))
    (do ((i 0 (+ i 1)))
        ((= i z) (unspecific))
      (string-set! v i x))))

(define (make-string=? char=?)
  (lambda (s1 s2)
    (let ((z (string-length s1)))
      (and (= z (string-length s2))
           (do ((i 0 (+ i 1))
                (result '()))
               ((boolean? result) result)
             (cond ((= i z) (set! result #t))
                   ((char=? (string-ref s1 i) (string-ref s2 i)))
                   (else (set! result #f))))))))

(define string=?    (make-string=? char=?))
(define string-ci=? (make-string=? char-ci=?))

(define (make-string<? char<? char=?)
  (lambda (s1 s2)
    (let ((z1 (string-length s1))
          (z2 (string-length s2)))
      (let ((z (min z1 z2)))
        (do ((i 0 (+ i 1))
             (result '()))
            ((boolean? result) result)
          (if (= i z)
              (set! result (< z1 z2))
              (let ((c1 (string-ref s1 i))
                    (c2 (string-ref s2 i)))
                (if (char<? c1 c2)
                    (set! result #t)
                    (if (not (char=? c1 c2))
                        (set! result #f))))))))))

(define string<?    (make-string<? char<? char=?))
(define string-ci<? (make-string<? char-ci<? char-ci=?))

(define (string>? s1 s2) (string<? s2 s1))
(define (string<=? s1 s2) (not (string>? s1 s2)))
(define (string>=? s1 s2) (not (string<? s1 s2)))

(define (string-ci>? s1 s2) (string-ci<? s2 s1))
(define (string-ci<=? s1 s2) (not (string-ci>? s1 s2)))
(define (string-ci>=? s1 s2) (not (string-ci<? s1 s2)))

(define (vector . l)
  (list->vector l))

(define (vector->list v)
  (do ((i (- (vector-length v) 1) (- i 1))
       (l '() (cons (vector-ref v i) l)))
      ((< i 0) l)))

(define (list->vector l)
  (let ((v (make-vector (length l) #f)))
    (do ((i 0 (+ i 1))
         (l l (cdr l)))
        ((null? l) v)
      (vector-set! v i (car l)))))

(define (vector-fill! v x)
  (let ((z (vector-length v)))
    (do ((i 0 (+ i 1)))
        ((= i z) (unspecific))
      (vector-set! v i x))))

(define (map proc first . rest)
  (if (null? rest)
      (map1 proc first)
      (map2+ proc first rest)))

(define (map1 proc l)
  (do ((l l (cdr l))
       (result '()))
      ((null? l) (reverse result))
    (set! result (cons (proc (car l)) result))))

(define (map2+ proc first rest)
  (do ((first  first (cdr first))
       (rest   rest  (map1 cdr rest))
       (result '()))
      ((or (null? first)
           (not (null? (filter null? rest)))) (reverse result))
    (set! result (cons (apply proc (cons (car first) (map1 car rest)))
                       result))))

(define (for-each proc first . rest)
  (do ((first first (cdr first))
       (rest  rest  (map1 cdr rest)))
      ((or (null? first)
           (and (not (null? rest))
                (not (null? (filter null? rest))))) (unspecific))
    (apply proc (cons (car first) (map car rest)))))

(define (inexact? n) (not (exact? n)))

(define (quotient n1 n2)
  (/ n1 n2))

(define (modulo x y)
  (let ((r (remainder x y)))
    (if (eq? (< r 0) (< y 0))
        r
        (+ r y))))

(define (ceiling x)
  (- 0 (floor (- 0 x))))

(define (truncate x)
  (if (< x 0)
      (ceiling x)
      (floor x)))

(define (round x)
  (let ((x+1/2 (+ x (/ 1 2))))
    (let ((r (floor x+1/2)))
      (if (and (= r x+1/2)
               (odd? r))
          (- r 1)
          r))))

(define (gcd . integers)
  (reduce (lambda (x y)
            (cond ((< x 0) (gcd (- 0 x) y))
                  ((< y 0) (gcd x (- 0 y)))
                  ((< x y) (euclid y x))
                  (else (euclid x y))))
          0
          integers))

(define (euclid x y)
  (if (= y 0)
      (if (and (inexact? y)
               (exact? x))
          (exact->inexact x)
          x)
      (euclid y (remainder x y))))

(define (lcm . integers)
  (reduce (lambda (x y)
            (let ((g (gcd x y)))
              (cond ((= g 0) g)
                    (else (* (quotient (abs x) g) (abs y))))))
          1
          integers))

(define (expt x n)
  (if (and (integer? n) (exact? n))
      (if (>= n 0)
          (raise-to-integer-power x n)
          (/ 1 (raise-to-integer-power x (- 0 n))))
      (exp (* n (log x)))))

(define (raise-to-integer-power x n)
  (if (= n 0)
      1
      (do ((s x (* s s))
           (i n)
           (a 1))
          ((= i 0) a)
        (set! a (if (odd? i) (* a s) a))
        (set! i (quotient i 2)))))

(define (newline . port)
  (apply write-char #\newline port))

(define (call-with-input-file filename proc)
  (let ((port #f))
    (dynamic-wind (lambda ()
                    (set! port (open-input-file filename)))
                  (lambda ()
                    (proc port))
                  (lambda ()
                    (if port (close-input-port port))))))

(define (call-with-output-file filename proc)
  (let ((port #f))
    (dynamic-wind (lambda ()
                    (set! port (open-output-file filename)))
                  (lambda ()
                    (proc port))
                  (lambda ()
                    (if port (close-output-port port))))))

(define (with-input-from-file filename thunk)
  (call-with-input-file filename
    (lambda (new-port)
      (let ((old-port (current-input-port)))
        (dynamic-wind (lambda ()
                        (set-current-input-port new-port))
                      thunk
                      (lambda ()
                        (set-current-input-port old-port)))))))

(define (with-output-to-file filename thunk)
  (call-with-output-file filename
    (lambda (new-port)
      (let ((old-port (current-output-port)))
        (dynamic-wind (lambda ()
                        (set-current-output-port new-port))
                      thunk
                      (lambda ()
                        (set-current-output-port old-port)))))))

(define (number->string n . r)
  (apply to-string n r))
(define (symbol->string s)
  (to-string s))

(define (null-environment)            '())
(define (scheme-report-environment r) '())
(define (interaction-environment)     '())

;; Mulitple value return hack
(define multiple-value-token (vector 'multiple-value-token))

(define (values . things)
  (if (and (pair? things)
           (null? (cdr things)))
      (car things)
      (cons multiple-value-token things)))

(define (call-with-values producer consumer)
  (let ((things (producer)))
    (if (and (pair? things)
             (eq? (car things) multiple-value-token))
        (apply consumer (cdr things))
        (consumer things))))

;; let
(define (specs? x)
  (or (null? x)
      (and (pair? x)
           (let ((s (car x)))
             (and (pair? s)
                  (symbol? (car s))
                  (pair? (cdr s))
                  (null? (cddr s))))
           (specs? (cdr x)))))

(define-rewriter 'let*
  (lambda (exp)
    (if (pair? (cdr exp))
        (let ((specs (cadr exp))
              (body (cddr exp)))
          (if (specs? specs)
              (if (or (null? specs)
                      (null? (cdr specs)))
                  (cons 'let
                        (cons specs body))
                  (cons 'let
                        (cons (list (car specs))
                              (cons (cons 'let* (cons (cdr specs) body))
                                    '()))))
              (error (string-append "Malformed variable bindings after let* "
                                    (to-string exp)))))
        (error (string-append "Expected list after let* "
                              (to-string exp))))))


;; quasiquote

(define (expand-quasiquote x level)
  (descend-quasiquote x level finalize-quasiquote))

(define (finalize-quasiquote mode arg)
  (cond ((eq? mode 'quote) (list 'quote arg))
        ((eq? mode 'unquote) arg)
        ((eq? mode 'unquote-splicing)
         (syntax-error ",@ in invalid context" arg))
        (else (cons mode arg))))

(define (descend-quasiquote x level return)
  (cond ((vector? x)
         (descend-quasiquote-vector x level return))
        ((not (pair? x))
         (return 'quote x))
        ((interesting-to-quasiquote? x 'quasiquote)
         (descend-quasiquote-pair x (+ level 1) return))
        ((interesting-to-quasiquote? x 'unquote)
         (cond ((= level 0)
                (return 'unquote (cadr x)))
               (else
                (descend-quasiquote-pair x (- level 1) return))))
        ((interesting-to-quasiquote? x 'unquote-splicing)
         (cond ((= level 0)
                (return 'unquote-splicing (cadr x)))
               (else
                (descend-quasiquote-pair x (- level 1) return))))
        (else
         (descend-quasiquote-pair x level return))))

(define (descend-quasiquote-pair x level return)
  (descend-quasiquote (car x) level
                      (lambda (car-mode car-arg)
                        (descend-quasiquote (cdr x) level
                                            (lambda (cdr-mode cdr-arg)
                                              (cond ((and (eq? car-mode 'quote) (eq? cdr-mode 'quote))
                                                     (return 'quote x))
                                                    ((eq? car-mode 'unquote-splicing)
                                                     ;; (,@mumble ...)
                                                     (cond ((and (eq? cdr-mode 'quote) (null? cdr-arg))
                                                            (return 'unquote
                                                                    car-arg))
                                                           (else
                                                            (return 'append
                                                                    (list car-arg (finalize-quasiquote
                                                                                   cdr-mode cdr-arg))))))
                                                    (else
                                                     (return 'cons
                                                             (list (finalize-quasiquote car-mode car-arg)
                                                                   (finalize-quasiquote cdr-mode cdr-arg))))))))))

(define (descend-quasiquote-vector x level return)
  (descend-quasiquote (vector->list x) level
                      (lambda (mode arg)
                        (if (eqv? mode 'quote)
                            (return 'quote x)
                            (return 'list->vector
                                    (list (finalize-quasiquote mode arg)))))))

(define (interesting-to-quasiquote? x marker)
  (and (pair? x) (eq? (car x) marker)))

(define-rewriter 'quasiquote
  (lambda (exp)
    (if (and (pair? (cdr exp))
             (null? (cddr exp)))
        (expand-quasiquote (cadr exp) 0)
        (error (string-append "Expected list after quasiquote "
                              (to-string exp))))))

(define-rewriter 'case
  (lambda (exp)
    (if (pair? (cdr exp))
        (let ((key (cadr exp))
              (clauses (cddr exp))
              (temp (gensym "temp")))
          `(let
               ((,temp ,key))
             (cond
              ,@(map (lambda (clause)
                       `(,(cond ((eq? (car clause) 'else)
                                 (car clause))
                                ((null? (car clause))
                                 #f)
                                ((null? (cdar clause)) ;+++
                                 `(eqv? ,temp (quote ,(caar clause))))
                                (else
                                 `(memv ,temp (quote ,(car clause)))))
                         ,@(cdr clause)))
                     clauses))))
        (error (string-append "Expected list after case "
                              (to-string exp))))))

(define-rewriter 'letrec
  (lambda (expr)
    (let ((specs (cadr expr))
          (body (cddr expr)))
      `((lambda ,(map car specs)
          ,@(map (lambda (spec) `(set! ,@spec)) (reverse specs))
          ((lambda () ,@body)))         ;allow internal defines
        ,@(map (lambda (spec)
                 spec                   ;ignored
                 ''unassigned)
               specs)))))

;; promises
(define make-promise
  (lambda (proc)
    (let ((result-ready? #f)
          (result #f))
      (lambda ()
        (if result-ready?
            result
            (let ((x (proc)))
              (if result-ready?
                  result
                  (begin (set! result-ready? #t)
                         (set! result x)
                         result))))))))

(define-rewriter 'delay
  (lambda (expr)
    (list 'make-promise `(lambda () . ,(cdr expr)))))

(define (force promise)
  (promise))

;; Load some Java environment specific bits
(load "bdc/scheme/util.scm")
